@@ -182,7 +182,9 @@ sub window_lost
    my $self = shift;
 
    foreach my $child ( $self->children ) {
-      $child->window->close;
+      my $childwin = $child->window;
+      $childwin and $childwin->close;
+
       $child->set_window( undef );
    }
 
