@@ -8,7 +8,7 @@ package Tickit::Widget;
 use strict;
 use warnings;
 
-our $VERSION = '0.51';
+our $VERSION = '0.52';
 
 use Carp;
 use Scalar::Util qw( weaken );
@@ -17,6 +17,7 @@ use List::Util 1.33 qw( all );
 use Tickit::Pen;
 use Tickit::Style;
 use Tickit::Utils qw( textwidth );
+use Tickit::Window 0.57;  # $win->bind_event
 
 use constant PEN_ATTR_MAP => { map { $_ => 1 } @Tickit::Pen::ALL_ATTRS };
 
@@ -513,32 +514,34 @@ sub window_gained
 
    weaken $self;
 
-   $window->set_on_geom_changed( sub {
+   my $event_ids = $self->{event_ids} //= {};
+
+   $event_ids->{geomchange} = $window->bind_event( geomchange => sub {
       $self->reshape;
       $self->redraw if !$self->parent;
    } );
 
-   $window->set_on_expose( sub {
-      my ( $win, $rb, $rect ) = @_;
+   $event_ids->{expose} = $window->bind_event( expose => sub {
+      my ( $win, undef, $info ) = @_;
       $win->is_visible or return;
 
-      $rb->setpen( $self->{pen} );
+      $info->rb->setpen( $self->{pen} );
 
-      $self->render_to_rb( $rb, $rect );
+      $self->render_to_rb( $info->rb, $info->rect );
    });
 
-   $window->set_on_focus( sub {
-      $self->_on_win_focus( @_ );
+   $event_ids->{focus} = $window->bind_event( focus => sub {
+      my ( $win, undef, $info ) = @_;
+      $self->_on_win_focus( $win, $info->type, $info->win );
    } ) if $self->can( "_widget_style_type" );
 
    if( $self->can( "on_key" ) or $self->KEYPRESSES_FROM_STYLE ) {
-      $window->set_on_key( sub {
-         shift;
-         my ( $ev ) = @_;
+      $event_ids->{key} = $window->bind_event( key => sub {
+         my ( $win, undef, $info ) = @_;
 
          {
             # Space comes as " " but we'd prefer to use "Space" in styles
-            my $keystr = $ev->str eq " " ? "Space" : $ev->str;
+            my $keystr = $info->str eq " " ? "Space" : $info->str;
 
             my $action;
             $action = $self->get_style_values( "<$keystr>" ) if $self->KEYPRESSES_FROM_STYLE;
@@ -548,18 +551,17 @@ sub window_gained
             last unless $action;
 
             my $code = $self->can( "key_$action" );
-            return 1 if $code and $code->( $self, @_ );
+            return 1 if $code and $code->( $self, $info );
          }
          my $code = $self->can( "on_key" );
-         return 1 if $code and $code->( $self, @_ );
+         return 1 if $code and $code->( $self, $info );
       } );
    }
 
-   $window->set_on_mouse( sub {
-      shift;
-      my ( $ev ) = @_;
-      $self->take_focus if $self->CAN_FOCUS and $ev->button == 1 and $ev->type eq "press";
-      $self->on_mouse( @_ ) if $self->can( "on_mouse" );
+   $event_ids->{mouse} = $window->bind_event( mouse => sub {
+      my ( $win, undef, $info ) = @_;
+      $self->take_focus if $self->CAN_FOCUS and $info->button == 1 and $info->type eq "press";
+      $self->on_mouse( $info ) if $self->can( "on_mouse" );
    } );
 }
 
@@ -591,11 +593,7 @@ sub window_lost
 
    my $window = $self->window;
 
-   $window->set_on_geom_changed( undef );
-   $window->set_on_expose( undef );
-   $window->set_on_focus( undef );
-   $window->set_on_key( undef );
-   $window->set_on_mouse( undef );
+   $window->unbind_event_id( $_ ) for values %{ $self->{event_ids} };
 }
 
 =head2 $window = $widget->window
