@@ -11,7 +11,7 @@ use base qw( Tickit::Widget::LinearSplit );
 use Tickit::Style;
 use Tickit::RenderBuffer qw( LINE_SINGLE CAP_BOTH );
 
-our $VERSION = '0.26';
+our $VERSION = '0.27';
 
 use Carp;
 
@@ -204,8 +204,10 @@ sub on_mouse
    my $self = shift;
    my ( $args ) = @_;
 
-   return unless $args->button == 1;
-   return $self->_on_mouse( $args->type, $args->line );
+   if( $args->type ne "wheel" and $args->button == 1 ) {
+      return $self->_on_mouse( $args->type, $args->line );
+   }
+   return;
 }
 
 =head1 AUTHOR
